//!
//! A micro Web framework based on Hyper, Futures and Tokio.
//!
//! # Example
//! ```{rust,ignore}
//! extern crate susanoo;
//!
//! use susanoo::prelude::*;
//!
//! fn hello(ctx: Context) -> Outcome<Response> {
//!     ctx.respond("Hello, world")
//! }
//!
//! fn main() {
//!     let s = Susanoo::default()
//!         .with_route(Route::get("/", hello));
//!     s.run().unwrap()
//! }
//! ```

pub extern crate futures;
pub extern crate hyper;
#[doc(hidden)]
pub extern crate url;

extern crate regex;
extern crate tokio_core;
extern crate tokio_proto;

pub mod body;
pub mod response;
pub mod router;

mod context;
mod error;
mod handler;
mod outcome;
mod susanoo;
mod storage;

// re-exports

#[doc(inline)]
pub use hyper::header;
#[doc(inline)]
pub use hyper::StatusCode;

#[doc(inline)]
pub use body::{Body, FromBody, FromForm};
pub use context::Context;
pub use error::{HandleError, ErrorHandler};
pub use handler::Handler;
pub use outcome::Outcome;
#[doc(inline)]
pub use response::{Response, Responder};
#[doc(inline)]
pub use router::Route;
pub use susanoo::Susanoo;
pub use storage::Storage;

pub mod prelude {
    pub use futures::{Future, Stream};
    pub use context::Context;
    pub use error::{HandleError, ErrorHandler};
    pub use handler::Handler;
    pub use outcome::Outcome;
    pub use response::Response;
    pub use router::Route;
    pub use susanoo::Susanoo;
}
