use std::collections::HashMap;
use hyper::Method;
use regex::RegexSet;

use super::route::Route;
use {Context, Handler, HandleError, Outcome, Response};

macro_rules! try_opt {
    ($e:expr) => {
        match $e {
            Some(v) => v,
            None => return None,
        }
    };
}


#[derive(Default, Clone)]
pub struct Router {
    routes: HashMap<Method, Vec<Route>>,
}

impl Router {
    pub(crate) fn add_route(&mut self, route: Route) -> &mut Self {
        self.routes
            .entry(route.method.clone())
            .or_insert(Vec::new())
            .push(route);
        self
    }

    // TODO: ranking
    fn find_route(&self, method: &Method, path: &str) -> Option<&Route> {
        let routes = try_opt!(self.routes.get(method));
        let set = RegexSet::new(routes.iter().map(|r| r.pattern_str())).unwrap();
        let matched = try_opt!(set.matches(path).into_iter().next());
        Some(&routes[matched])
    }
}

impl Handler for Router {
    fn call(&self, mut ctx: Context) -> Outcome<Response> {
        match self.find_route(&ctx.method(), &ctx.path()) {
            Some(route) => {
                let params = route.pattern.owned_captures(&ctx.path()).unwrap();
                ctx.states_mut().set(params);
                route.handler.call(ctx)
            }
            None => Outcome::err(HandleError::not_found()),
        }
    }
}
